/// This will force a panic to occur with the given arguments.
/// This is mostly used from the error! macro.
#[doc(hidden)]
#[macro_export]
macro_rules! generate_panic
{
   ($($arg: tt)+) =>
   ({
      panic!(::std::fmt::format(format_args!($($arg)+)));
   })
}

/// This handles turning a module path into a domain name.
#[doc(hidden)]
#[macro_export]
macro_rules! determine_domain
{
   () =>
   ({
      match module_path!().find(":")
      {
         Some(pos) =>
         {
            &module_path!()[0..pos]
         }

         None =>
         {
            module_path!()
         }
      }
   });
}

/// This handles calling the actual logging function.
#[doc(hidden)]
#[macro_export]
macro_rules! log
{
   (domain: $domain_name: expr, $log_level: expr, $($arg: tt)+) =>
   ({
      $crate::log_record($domain_name, $log_level, module_path!(),
                         file!(), line!(), format_args!($($arg)+));
   });

   ($log_level: expr, $($arg: tt)+) =>
   ({
      log!(domain: determine_domain!(), $log_level, $($arg)+);
   });
}

/// Logs a message at the Error severity Level.
///
/// This is for a serious, program stopping, problem.
#[macro_export]
macro_rules! error
{
   (domain: $domain_name: expr, $($arg: tt)*) =>
   ({
      log!(domain: $domain_name, $crate::Levels::Error, $($arg)*);
      generate_panic!($($arg)*);
   });

   ($($arg: tt)*) =>
   ({
      log!($crate::Levels::Error, $($arg)*);
      generate_panic!($($arg)*);
   });
}

/// Logs a message at the Warn severity Level.
///
/// This is for a problem that will not stop the
/// program from running.
#[macro_export]
macro_rules! warn
{
   (domain: $domain_name: expr, $($arg: tt)*) =>
   ({
      log!(domain: $domain_name, $crate::Levels::Warn, $($arg)*);
   });

   ($($arg: tt)*) =>
   ({
      log!($crate::Levels::Warn, $($arg)*);
   });
}

/// Logs a message at the Info severity Level.
///
/// This is for general useful information.
#[macro_export]
macro_rules! info
{
   (domain: $domain_name: expr, $($arg: tt)*) =>
   ({
      log!(domain: $domain_name, $crate::Levels::Info, $($arg)*);
   });

   ($($arg: tt)*) =>
   ({
      log!($crate::Levels::Info, $($arg)*);
   })
}

/// Logs a message at the Debug severity Level.
///
/// This is for debugging only information.
///
/// Logging at this level is disabled when in release mode.
#[macro_export]
macro_rules! debug
{
   (domain: $domain_name: expr, $($arg: tt)*) =>
   ({
      if cfg!(debug_assertions) == true
      {
         log!(domain: $domain_name, $crate::Levels::Debug, $($arg)*);
      }
   });

   ($($arg: tt)*) =>
   ({
      // Only log this if we are currently in a debug build.
      if cfg!(debug_assertions) == true
      {
         log!($crate::Levels::Debug, $($arg)*);
      }
   });
}
