use ::levels::Levels;
use ::location::Location;



/// A single message to be logged.
pub struct Record<'a>
{
   /// The domain that the message is supposed to be
   /// logged to.
   domain: &'a str,

   /// The severity Level for this message.
   level: Levels,

   /// The Location that the message was generated at.
   location: Location<'a>,

   /// The message to be logged.
   message: ::std::fmt::Arguments<'a>
}



impl<'a> Record<'a>
{
   /// Create a new Record.
   pub fn new(domain_name: &'a str, lvl: Levels,
              message_data: ::std::fmt::Arguments<'a>)
      -> Record<'a>
   {
      Record
      {
         domain: domain_name,
         level: lvl,
         location: Location::new("", "", 0),
         message: message_data
      }
   }

   /// Get the domain of this Record.
   pub fn get_domain(&self) -> &str
   {
      self.domain
   }

   /// Get the Level of this Record.
   pub fn get_level(&self) -> Levels
   {
      self.level
   }

   /// Get the Location where this Record was generated.
   pub fn get_location(&self) -> &Location
   {
      &self.location
   }

   /// Get the message that this Record holds.
   pub fn get_message(&self) -> ::std::fmt::Arguments<'a>
   {
      self.message
   }

   /// Set the domain that this Record is to be logged to.
   pub fn set_domain(&mut self, domain_name: &'a str)
   {
      self.domain = domain_name;
   }

   /// Set the severity Level of the Record.
   pub fn set_level(&mut self, lvl: Levels)
   {
      self.level = lvl;
   }

   /// Set the Location where this Record was generated.
   pub fn set_location(&mut self, module: &'a str, file: &'a str,
                       line: u32)
   {
      self.location = Location::new(module, file, line);
   }

   /// Set the message for this Record.
   pub fn set_message(&mut self, message_data: ::std::fmt::Arguments<'a>)
   {
      self.message = message_data;
   }
}

impl<'a> ::std::fmt::Debug for Record<'a>
{
   fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result
   {
      write!(f, "{}", self)
   }
}

impl<'a> ::std::fmt::Display for Record<'a>
{
   fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result
   {
      write!(f, "{}-{}:  {}  --  {}", self.get_domain(), self.get_level(),
             self.get_location(), self.get_message())
   }
}
